package com.jh.common.security.service;

import java.util.Date;
import java.util.HashSet;
import java.util.Set;
import java.util.UUID;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import com.jh.common.email.EmailService;
import com.jh.common.security.AuthService;
import com.jh.common.security.GrantedRole;
import com.jh.common.security.PasswordHash;
import com.jh.common.security.PasswordUtils;
import com.jh.common.security.model.AppUser;
import com.jh.common.security.model.ResetPasswordToken;
import com.jh.common.security.repository.AppUserRepository;



public class AppUserAuthService implements AuthService {

	private static final Logger LOG = LoggerFactory.getLogger(AppUserAuthService.class);
	
	private AppUserRepository appUserRepository;
	
	private EmailService emailService;
	

	@Override
	@Transactional
	public Authentication authenticate(String login, String password) throws BadCredentialsException {
		AppUser user = appUserRepository.findByLogin(login);
		if (user == null) {
			throw new BadCredentialsException("User not found");
		}
		if (!PasswordUtils.checkPassword(password, user.getPassword(), user.getPasswordSalt())) {
			throw new BadCredentialsException("Bad password");
		}
		
		Set<GrantedRole> roles = new HashSet<>();
		
		UsernamePasswordAuthenticationToken auth = new UsernamePasswordAuthenticationToken(user.getEmail(), user.getPassword(), roles);
		return auth;
	}

	@Override
	@Transactional
	public void register(String login, String password) throws AuthenticationException {
		AppUser user = appUserRepository.findByLogin(login);
		if (user != null) {
			throw new BadCredentialsException("User already exists");
		}
		if (!StringUtils.hasText(login) || !StringUtils.hasText(password)) {
			throw new BadCredentialsException("Bad username or password");
		}
		appUserRepository.registerUser(login, password);
	}

	@Override
	@Transactional
	public String generateResetToken(String login) {
		AppUser user = appUserRepository.findByLogin(login);
		if (user == null) {
			throw new BadCredentialsException("User does not exist");
		}
		
		ResetPasswordToken token = new ResetPasswordToken(login, new Date(), UUID.randomUUID().toString());
		appUserRepository.saveResetPasswordToken(token);
		
		//TODO send email
		if (emailService == null) {
			LOG.warn("Reset password email cannot be sent: no email service configured");
		} else {
			
		}
		
		return token.getToken();
	}

	
	@Override
	@Transactional
	public void resetPassword(String login, String token, String newPassword) throws AuthenticationException {
		ResetPasswordToken rpt = appUserRepository.findResetPasswordToken(login, token);
		if (rpt == null) {
			throw new BadCredentialsException("Invalid token");
		}
	
		AppUser user = appUserRepository.findByLogin(login);
		if (user == null) {
			throw new BadCredentialsException("User does not exist");
		}
		
		PasswordHash hash = PasswordUtils.hashPassword(newPassword);
		user.setPassword(hash.getHash());
		user.setPasswordSalt(hash.getSalt());
	}
	
	
	@Autowired(required = false)
	public void setAppUserRepository(AppUserRepository appUserRepository) {
		this.appUserRepository = appUserRepository;
	}

	@Autowired(required = false)
	public void setEmailService(EmailService emailService) {
		this.emailService = emailService;
	}
	
}
